import numpy as np
from abc import abstractmethod, ABC

class Cache(ABC):
	@abstractmethod
	def set(self, key:str, value:np.ndarray):
		pass
	
	@abstractmethod
	def get(self, key:str) -> np.ndarray:
		pass
	
	@abstractmethod
	def check(self, key:str) -> bool:
		pass


import os
import numpy as np
import random
import string
from overrides import overrides
from pathlib import Path
from typing import Any
from .cache import Cache

class NpyFS(Cache):
	def __init__(self, baseDirectory=None):
		if baseDirectory is None:
			Str = "".join(random.choices(string.printable, k=10))
			baseDirectory = ".cache/%s" % Str

		self.cacheDir = Path(os.path.abspath(os.path.realpath(baseDirectory)))
		self.cacheDir.mkdir(exist_ok=True, parents=True)
		print("[NpyFS] Cache dir: %s" % str(self.cacheDir))

	@overrides
	def get(self, key:str) -> np.ndarray:
		if not key.endswith(".npy"):
			key = "%s.npy" % key
		cacheFile = Path("%s/%s" % (str(self.cacheDir), key))

		item = np.load(cacheFile, allow_pickle=True)
		try:
			item = item.item()
		except Exception:
			pass
		return item

	@overrides
	def check(self, key:str) -> bool:
		if not key.endswith(".npy"):
			key = "%s.npy" % key
		cacheFile = Path("%s/%s" % (str(self.cacheDir), key))
		return cacheFile.exists() and cacheFile.is_file()
	
	@overrides
	def set(self, key:str, value:np.ndarray):
		if not key.endswith(".npy"):
			key = "%s.npy" % key
		cacheFile = Path("%s/%s" % (str(self.cacheDir), key))
		cacheFile.parent.mkdir(exist_ok=True, parents=True)
		np.save(cacheFile, value)

	def __str__(self) -> str:
		Str = "[NpyFS]"
		Str += "\n  - Cache directory: %s" % str(self.cacheDir)
		return Str


import numpy as np
from overrides import overrides
from pathlib import Path
from typing import Any
from .cache import Cache

class DictMemory(Cache):
	def __init__(self):
		self.cache = {}

	@overrides
	def get(self, key:str) -> np.ndarray:
		key = key.split("/")
		item = self.cache
		for i in range(len(key)):
			item = item[key[i]]
		return item

	@overrides
	def check(self, key:str) -> bool:
		key = key.split("/")
		item = self.cache
		for i in range(len(key)):
			if not key[i] in item:
				return False
			item = item[key[i]]
		return True
	
	@overrides
	def set(self, key:str, value:np.ndarray):
		key = key.split("/")
		item = self.cache
		for i in range(len(key) - 1):
			if not key[i] in item:
				item[key[i]] = {}
			item = item[key[i]]
		item[key[-1]] = value

	def __str__(self) -> str:
		Str = "[DictMemory]"
		Str += "\n  - Num top level keys: %d" % len(self.cache.keys())
		return Str


from simple_caching import NpyFS

class TestNpyFS:
    def test_npyfs_1(self):
        cache = NpyFS()
        assert not cache is None

    def test_npyfs_2(self):
        cache = NpyFS()
        assert cache.check("hi") == False
        try:
            _ = cacge.get("hi")
            assert False
        except Exception:
            pass
        cache.set("hi", 3)
        assert cache.check("hi") == True
        assert cache.get("hi") == 3


import setuptools
import os

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

try:
    requirementsFilePath = os.path.dirname(os.path.abspath(__file__)) + "/requirements.txt"
    requirements = open(requirementsFilePath, "r").read().splitlines()
except Exception:
    requirements = []

setuptools.setup(
    name="python-simple-caching",
    version="0.12",
    author="Mihai Cristian Pîrvu",
    author_email="mihaicristianpirvu@gmail.com",
    description="Simple caching library for python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/mihaicristianpirvu/simple-caching/",
    keywords = ["caching", "cache"],
    packages=setuptools.find_packages(),
    install_requires=requirements,
    license="WTFPL",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)
